(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     10067,        330]
NotebookOptionsPosition[      7439,        245]
NotebookOutlinePosition[      9049,        291]
CellTagsIndexPosition[      8965,        286]
WindowTitle->BaranovMortality - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["EconMult",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:EconMult/guide/EconMult"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["ECONMULT PACLET SYMBOL", "PacletNameCell"], Cell[TextData[Cell[
    BoxData[
     ActionMenuBox[
      FrameBox["\<\"URL \[RightGuillemet]\"\>",
       StripOnInput->False], {"\<\"EconMult/ref/BaranovMortality\"\>":>
      None, "\<\"Copy Mathematica url\"\>":>
      Module[{DocumentationSearch`Private`nb$}, 
       DocumentationSearch`Private`nb$ = NotebookPut[
          Notebook[{
            Cell["EconMult/ref/BaranovMortality"]}, Visible -> False]]; 
       SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
       FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
       NotebookClose[DocumentationSearch`Private`nb$]; Null], 
      Delimiter, "\<\"Copy web url\"\>":>
      Module[{DocumentationSearch`Private`nb$}, 
       DocumentationSearch`Private`nb$ = NotebookPut[
          Notebook[{
            Cell[
             BoxData[
              MakeBoxes[
               Hyperlink[
               "http://reference.wolfram.com/mathematica/EconMult/ref/\
BaranovMortality.html"], StandardForm]], "Input", TextClipboardType -> 
             "PlainText"]}, Visible -> False]]; 
       SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
       FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
       NotebookClose[DocumentationSearch`Private`nb$]; 
       Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
        NotebookLocate[{
          URL[
           StringJoin[
           "http://reference.wolfram.com/mathematica/", 
            "EconMult/ref/BaranovMortality", ".html"]], None}]}]},
      Appearance->None,
      MenuAppearance->Automatic]],
     LineSpacing->{1.4, 0}]], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["BaranovMortality", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"BaranovMortality", "[", "t", "]"}]], "InlineFormula"],
     " \[LineSeparator]gives the differential equation of number of individs \
as a function of age (t)."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->163260964],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->308117975],

Cell[BoxData[
 RowBox[{
  RowBox[{"Needs", "[", "\"\<EconMult`PopulationGrowth`\>\"", "]"}], 
  ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->59286962],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BaranovMortality", "[", 
  RowBox[{"t", ",", " ", 
   RowBox[{"Fishing", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->153999500],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["BaranovNumbers", "\[Prime]",
    MultilineFunction->None], "[", "t", "]"}], "\[Equal]", 
  RowBox[{
   RowBox[{"-", "PGM"}], " ", 
   RowBox[{"BaranovNumbers", "[", "t", "]"}]}]}]], "Output",
 ImageSize->{296, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->373313887]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Notation", "@", 
  RowBox[{"BaranovMortality", "[", 
   RowBox[{"t", ",", " ", 
    RowBox[{"Fishing", "\[Rule]", "False"}]}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->302990283],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["\<\"N\"\>", "\[Prime]",
      MultilineFunction->None], "(", "t", ")"}], "\[LongEqual]", 
    RowBox[{
     RowBox[{"-", "\<\"M\"\>"}], " ", 
     RowBox[{"\<\"N\"\>", "[", "t", "]"}]}]}],
   Function[BoxForm`e$, 
    MatrixForm[BoxForm`e$]]], TraditionalForm]], "Output",
 ImageSize->{88, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]//TraditionalForm=",
 CellID->198314646]
}, Open  ]],

Cell["\<\
Number of individuals as a function of age (t) is found by solving the \
differential equation above:\
\>", "ExampleText",
 CellID->430166258],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DSolve", "[", 
   RowBox[{
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"BaranovMortality", "[", 
       RowBox[{"t", ",", " ", 
        RowBox[{"Fishing", "\[Rule]", "False"}]}], "]"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"BaranovNumbers", "[", "PGtR", "]"}], "\[Equal]", 
       RowBox[{"Recruitment", "[", "PGSB", "]"}]}]}], "\[IndentingNewLine]", 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"BaranovNumbers", "[", "t", "]"}], ",", "\[IndentingNewLine]", 
    "t"}], "\[IndentingNewLine]", "]"}], "//", "SimplifyNotation"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->170236062],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{"(", "\[NoBreak]", GridBox[{
      {
       RowBox[{
        RowBox[{"\<\"N\"\>", "(", "t", ")"}], "\[Rule]", 
        RowBox[{"\<\"R\"\>", " ", 
         SuperscriptBox["\[ExponentialE]", 
          RowBox[{"\<\"M\"\>", " ", 
           RowBox[{"(", 
            RowBox[{
             SubscriptBox["\<\"t\"\>", "\<\"R\"\>"], "-", "t"}], ")"}]}]]}]}]}
     },
     GridBoxAlignment->{
      "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
        "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.7]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
   Function[BoxForm`e$, 
    MatrixForm[BoxForm`e$]]], TraditionalForm]], "Output",
 ImageSize->{109, 17},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]//TraditionalForm=",
 CellID->180571889]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"BaranovMortality - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2011, 4, 7, 10, 47, 27.3752129}", "context" -> "EconMult`", 
    "keywords" -> {}, "index" -> True, "label" -> "EconMult Paclet Symbol", 
    "language" -> "en", "paclet" -> "EconMult", "status" -> "None", "summary" -> 
    "BaranovMortality[t] gives the differential equation of number of \
individs as a function of age (t).", "synonyms" -> {}, "title" -> 
    "BaranovMortality", "type" -> "Symbol", "uri" -> 
    "EconMult/ref/BaranovMortality"}, "LinkTrails" -> "", 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (February 23, 2011)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "8.0 for Microsoft Windows (64-bit) (February 23, 2011)",
   StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[3307, 97, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->163260964]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 8823, 279}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[609, 21, 76, 1, 70, "SymbolColorBar"],
Cell[688, 24, 179, 5, 70, "LinkTrail"],
Cell[870, 31, 1890, 42, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[2785, 77, 59, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[2847, 80, 423, 12, 70, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[3307, 97, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->163260964],
Cell[CellGroupData[{
Cell[3793, 114, 148, 5, 70, "ExampleSection",
 CellID->308117975],
Cell[3944, 121, 154, 5, 70, "Input",
 CellID->59286962],
Cell[CellGroupData[{
Cell[4123, 130, 180, 5, 70, "Input",
 CellID->153999500],
Cell[4306, 137, 376, 12, 36, "Output",
 CellID->373313887]
}, Open  ]],
Cell[CellGroupData[{
Cell[4719, 154, 212, 6, 70, "Input",
 CellID->302990283],
Cell[4934, 162, 499, 16, 50, "Output",
 CellID->198314646]
}, Open  ]],
Cell[5448, 181, 152, 4, 70, "ExampleText",
 CellID->430166258],
Cell[CellGroupData[{
Cell[5625, 189, 679, 17, 70, "Input",
 CellID->170236062],
Cell[6307, 208, 1066, 30, 52, "Output",
 CellID->180571889]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[7412, 243, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

